use chrono::Local;
use eframe::egui::{Color32, Context, Label, Response, RichText, Ui, Widget};
use eframe::{egui, Frame};
use serde::{Deserialize, Serialize};
use std::default;
use std::ops::Sub;
use std::time::{Duration, Instant, SystemTime, UNIX_EPOCH};

use crate::prelude::CanonicalName;

#[derive(Serialize, Deserialize)]
pub struct MonitorTarget {
    url: String,
    last_update: u128,
    last_status: bool,
}

impl MonitorTarget {
    pub fn new<S: Into<String>>(url: S) -> MonitorTarget {
        MonitorTarget {
            url: url.into(),
            last_status: false,
            last_update: Local::now().timestamp_millis() as u128,
        }
    }

    pub fn check(&mut self) -> bool {
        if (Local::now().timestamp_millis() as u128 - self.last_update) > 30 {
            self.last_status = reqwest::blocking::get(self.url.as_str()).is_ok();
            self.last_update = Local::now().timestamp_millis() as u128;
        }
        self.last_status
    }
}

#[derive(Default)]
pub struct UptimeWidget {
    pub targets: Vec<MonitorTarget>,
    _url_input_cache: String,
}

impl CanonicalName for UptimeWidget {
    fn canonical_name() -> String {
        "Uptimes".into()
    }
}

impl eframe::App for UptimeWidget {
    fn update(&mut self, ctx: &Context, frame: &mut Frame) {
        egui::Window::new("Uptime Settings")
            .default_open(false)
            .show(ctx, |ui| {
                self.targets.retain(|t| {
                    ui.add_space(1.);
                    ui.horizontal(|ui| {
                        ui.style_mut().visuals.extreme_bg_color = Color32::DARK_GRAY;
                        ui.label(t.url.clone());
                        !ui.button("remove").clicked()
                    })
                    .inner
                });
                ui.add_space(2.);
                ui.horizontal(|ui| {
                    ui.text_edit_singleline(&mut self._url_input_cache)
                        .on_hover_text("URL to add");
                    if ui.button("Add URL").clicked() {
                        self.targets
                            .push(MonitorTarget::new(self._url_input_cache.clone()));
                        self._url_input_cache = String::default();
                    }
                });
            });

        self.targets.iter_mut().for_each(|t| {
            egui::Window::new(t.url.clone())
                .title_bar(false)
                .movable(true)
                .resizable(true)
                .show(ctx, |ui| {
                    ui.vertical_centered(|ui| {
                        ui.label(t.url.clone());
                        ui.label({
                            if ctx.is_using_pointer() {
                                RichText::new("...")
                            } else if t.check() {
                                RichText::new("Up").color(Color32::LIGHT_GREEN)
                            } else {
                                RichText::new("Down")
                                    .color(Color32::LIGHT_RED)
                                    .strong()
                            }.size(32.)
                        });
                    });
                });
        });

        if self.targets.len() > 0 {
            ctx.request_repaint();
        }
    }
}
