use std::collections::HashMap;

use crate::apps::clock::ClockWidget;
use crate::apps::greeting::Greeting;
use crate::apps::todo::TodoWidget;
use crate::apps::uptime::UptimeWidget;
use crate::prelude::CanonicalName;
use eframe::egui::accesskit::Role::Window;
use eframe::egui::panel::Side;
use eframe::egui::{CentralPanel, Id, SidePanel};
use eframe::{egui, App};
use serde::{Deserialize, Serialize};

#[derive(Default)]
pub struct Dashboard {
    pub user_name: String,
    pub apps: HashMap<String, Box<dyn App>>,
}

impl Dashboard {
    pub fn has_app<S: Into<String>>(&self, name: S) -> bool {
        self.apps.contains_key(&name.into())
    }

    pub fn add_app<S: Into<String>>(&mut self, name: S, app: Box<dyn App>) {
        self.apps.insert(name.into(), app);
    }

    pub fn remove_app<S: Into<String>>(&mut self, name: S) {
        self.apps.remove(&name.into());
    }
}

macro_rules! register_app {
    ($dash:ident, $ui:ident, $app:tt) => {
        let button = $ui.button($app::canonical_name());
        if button.clicked() {
            if $dash.has_app($app::canonical_name()) {
                $dash.remove_app($app::canonical_name());
            } else {
                $dash.add_app($app::canonical_name(), Box::new($app::default()));
            }
        }
        if $dash.has_app($app::canonical_name()) {
            button.highlight();
        }
    };
}

impl eframe::App for Dashboard {
    fn update(&mut self, ctx: &egui::Context, frame: &mut eframe::Frame) {
        SidePanel::new(Side::Left, Id::new("side_panel")).show(ctx, |ui| {
            ui.heading("Apps");
            register_app!(self, ui, ClockWidget);
            register_app!(self, ui, UptimeWidget);
            register_app!(self, ui, TodoWidget);
        });
        CentralPanel::default().show(ctx, |_| {});
        self.apps.iter_mut().for_each(|(_, app)| {
            app.update(ctx, frame);
        })
    } 
}
